import { EventEmitter } from "events";

function validateEvt(evtInstance) {
  if (!evtInstance) {
    throw new Error("invalid evt instance");
  }

  if (!evtInstance instanceof EventEmitter) {
    throw new Error("Invalid evt instance type");
  }
}

export default class {
  constructor() {}

  set evt(evtInstance) {
    if (evtInstance instanceof EventEmitter) {
      this.__evt = evtInstance;
      return;
    }
    this.__evt = new EventEmitter();
  }

  get evt() {
    return this.__evt;
  }

  on(evtName, handler) {
    validateEvt(this.evt);
    if (!handler instanceof Function) {
      throw new Error("event handler must be a function");
    }
    this.evt.on(evtName, handler);
  }

  emit(...args) {
    validateEvt(this.evt);
    this.evt.emit.apply(this.evt, args);
  }

  removeListener(...args) {
    return this.evt.removeListener.apply(this.evt, args);
  }

  removeAllListeners(...args) {
    return this.evt.removeAllListeners.apply(this.evt, args);
  }
}
