try {
  if (!window.Promise) window.Promise = Promise;
} catch (err) {}

export default class {
  promiseCall(param) {
    if (typeof param !== "function") {
      //传入参数不是函数
      return new Promise(resolve => {
        resolve(param);
      });
    }
    return new Promise((resolve, reject) => {
      try {
        resolve(param.bind(this)());
      } catch (err) {
        reject(err);
      }
    });
  }

  /**
   * 基于Promise的延迟方法
   * 调用方法：this.delay(1000).then(...)
   * @param {Number} duration 等待的时间，单位毫秒
   * @return {Promise}
   */
  delay(duration) {
    return new Promise(resolve => {
      setTimeout(resolve, duration);
    });
  }

  /**
   * Promise.all 的节流版本，应对场景为大量并行异步操作，但终端无法支持的情况。
   * 如瞬时大批量的api调用，但服务器有并发限制，导致部分并发接口返回错误
   *
   * @param {Array} paramList 参数列表
   * @param {Function} asynchandler 批量处理的handler函数
   * @param {number} [threshold=5] 节流阈值，即一次的并发请求个数
   * @return {Promise} 多次Promise.all返回的结果集合
   */
  throttlePromiseAll(paramList, asynchandler, threshold = 5) {
    if (!asynchandler instanceof Function) {
      throw new Error("Invalid promise Handler type");
    }

    if (threshold === 0) {
      throw new Error("threshold can not be 0");
    }

    let newStack = [];

    let cache = [];

    paramList.forEach((param, ind) => {
      if (cache.length === threshold) {
        newStack.push(cache);
        cache = [];
      }
      cache.push(param);
    });

    if (cache.length > 0) {
      newStack.push(cache);
    }

    return newStack.reduce((prePromise, curThreads) => {
      return prePromise.then(lastBatchRst => {
        return Promise.all(curThreads.map(asynchandler)).then(rst => {
          return lastBatchRst.concat(rst);
        });
      });
    }, Promise.resolve([]));
  }
}
