const getValByKey = (instance, attr) => {
  if (instance === undefined || instance === null) {
    return instance;
  }
  if (typeof instance !== "object") {
    return instance;
  }
  if (!attr) {
    return instance;
  }
  return instance[attr];
};

export default class {
  /**
   * 找出sourceList中，比targetList多的元素，比较标准为fieldName
   * 如“找出sourcelist中，所有name属性不属于targetlist中所有元素name属性的元素，name就是fieldName”
   *
   * @param {Array} sourceList 源list
   * @param {Array} targetList 目标list
   * @param {string} fieldName 属性名
   * @return {Array} sourceList中比targetList多的元素
   */
  static getMoreByKey(sourceList, targetList, fieldName) {
    return sourceList.filter(item => {
      //筛选，先记录当前name
      let searchVal = getValByKey(item, fieldName);

      //尝试在targetList中寻找同名item
      let searchTargetWithSameName = targetList.find(instance => getValByKey(instance, fieldName) === searchVal);

      if (!searchTargetWithSameName) {
        //若未找到，则该item需要被处理
        return true;
      }
      //否则找到，说明target内相应元素已创建，不需要重新创建
      return false;
    });
  }

  /**
   * 遍历对象所有属性，针对每个属性调用一次处理函数
   *
   * @param {Object} targetObj 需要遍历的对象
   * @param {Function} handler 针对每个属性需要调用的函数，函数接受2个参数：key,value
   */
  static traverseObj(targetObj, handler) {
    if (!handler instanceof Function) {
      throw new Error("Handler has to be a function");
    }

    for (let key in targetObj) {
      handler(key, targetObj[key]);
    }
  }

  /**
   * 将一个对象序列化为字符串，格式类似于http的query格式如：a=1&b=2&c=3
   * 主要用于整理http的post请求，有些接口需要将body参数格式化为string传出
   *
   * @param {Object} jsonObj 目标对象
   * @param {string} [seperator="&"] 参数之间的分隔符
   * @return {string} 经过处理的字符串
   */
  static convertJsonToStr(jsonObj, seperator = "&") {
    let output = [];
    this.traverseObj(jsonObj, (key, val) => {
      output.push(`${key}=${val}`);
    });
    return output.join(seperator);
  }
}
