const { Base, IEvent } = require("../dist/kwm.node.base");
const Rst = require("kwm.rst");
const Assert = require("assert");

describe("Base Class", () => {
  it("rst instance", () => {
    let instance = new Base();

    let flag = !!instance.rst.err200;

    Assert.strictEqual(flag, true);
  });

  it("evt instance", () => {
    let instance = new Base();

    let flag = !!instance.evt.on && !!instance.evt.emit;

    Assert.strictEqual(flag, true);
  });

  it("promisefy", () => {
    let instance = new Base();

    return instance.promiseCall(() => {
      Assert.strictEqual(1, 1);
    });
  });

  it("IEvent", () => {
    class Test extends IEvent {
      constructor() {
        super();
        this.evt = "";
      }
    }

    let instance = new Test();

    let flag = !!instance.evt.on && !!instance.evt.emit;

    Assert.strictEqual(flag, true);
  });

  it("Evt passing", () => {
    class Member extends Base {
      constructor(evt) {
        super(evt);
      }
    }
    class Container extends Base {
      constructor(evt) {
        super(evt);
        this.member = new Member(this.evt);
      }
    }

    let container = new Container();
    if (!container.evt || !container.member.evt) {
      return Assert.fail("empty evt obj");
    }

    Assert.strictEqual(container.evt, container.member.evt);
  });

  it("Evt passing 2", () => {
    class Member extends Base {
      constructor(evt) {
        super(evt);
      }
    }
    class Container extends Base {
      constructor(evt) {
        super(evt);
        this.member = new Member();
      }
    }

    let container = new Container();

    if (!container.evt || !container.member.evt) {
      return Assert.fail("empty evt obj");
    }

    Assert.notEqual(container.evt, container.member.evt);
  });

  it("throttle Promise All", () => {
    class Sample extends Base {
      constructor(evt) {
        super(evt);
      }
    }

    let sample = new Sample();
    let startTime = new Date();

    let paramList = [
      { delay: 20, out: 1 },
      { delay: 50, out: 2 },
      { delay: 100, out: 3 },
      { delay: 30, out: 4 },
      { delay: 15, out: 5 }
    ];

    return sample
      .throttlePromiseAll(
        paramList,
        input => {
          return Promise.resolve()
            .then(() => {
              console.log(`func ${input.out} started at:${new Date() - startTime} `);
            })
            .then(() => {
              return sample.delay(input.delay);
            })
            .then(() => {
              console.log(`func ${input.out} finished at:${new Date() - startTime}`);
              return input.out;
            });
        },
        1
      )
      .then(rst => {
        console.log(rst);
        Assert.ok(rst);
      });
  });

  it("getMoreByKey method", () => {
    class tmp extends Base {
      constructor(evt) {
        super(evt);
      }
    }

    let rst = tmp.getMoreByKey([1, 2, 3, 4, 5], [{ age: 1 }, { age: 2 }, { age: 3 }, { age: 4 }], "age");
    Assert.ok(rst.join("") === "5");
  });

  it("convertJsonToStr method", () => {
    class tmp extends Base {
      constructor(evt) {
        super(evt);
      }
    }

    let rst = tmp.convertJsonToStr({ a: 1, b: 2, c: 3 }, ",");
    Assert.ok(rst === "a=1,b=2,c=3");
  });
});
